# HTTP基础








<extoc></extoc>

## HTTP常用头字段

| 字段名             | 方向     | 解释                                                         | 可能的值                                                     |
| ------------------ | -------- | ------------------------------------------------------------ | ------------------------------------------------------------ |
| Accept             | Request  | 接受什么介质类型                                             | type/sub-type<br />`*/*` 表示任何类型 , type/* 表示该类型下的所有子类型 |
| Accept-Charset     | Request  | 接收的字符集                                                 | ISO-8859-1                                                   |
| Accept-Encoding    | Request  | 接收的编码方法 , 通常指定压缩方法 , 是否支持压缩 , 支持什么压缩方法 | Gzip , deflate , UTF8                                        |
| Accept-Language    | Request  | 接收的语言                                                   | En , cn                                                      |
| Cache-Control      | Request  | 对服务器的缓存控制                                           | no-cache : 不要从缓存中去取 , 要求现在从Web服务器中去取      |
| Connection         | Request  | 连接状态通知                                                 | Close : 告诉Web服务器在完成本次请求的响应后 , 断开连接 , 不要等待本次连接的后续请求了<br />Keepalive : 告诉Web服务器在完成本次请求的响应后 , 保持连接 , 等待本次连接的后续请求 |
| Host               | Request  | 客户端指定自己想访问的Web服务器的域名 , IP 地址和端口号      | IP : port                                                    |
| Proxy-Authenticate | Request  | 提供自己在代理服务器中的身份信息                             | Username : password                                          |
| range              | Request  | 需要获取对象的哪一部分内容                                   | bytes=1024-    : 获取从第1024个字节到最后的内容              |
| Referer            | Request  | 浏览器向 Web 服务器表明自己是从哪个 URL 获得当前请求中的 URL 的 | http://www.baidu.com                                         |
| User-Agent         | Request  | 指明浏览器的软件类型及版本                                   | Mozilla/x.x: Windows浏览器<br />Firefox/xx.x.x: Fiefox浏览器 |
| Age                | Response | 用该头部表示该实体从产生到现在经过多长时间                   |                                                              |
| Authorization      | Response | 当客户端接收到来 自 Web 服务器的WWW-Authenticate响应时 , 该头部回应自己的身份验证信息给Web服务器 | Username : password                                          |
| Cache-Control      | Response | 对客户端的缓存控制                                           | Public : 可以用缓存内容回应任何用户<br />Private : 只能用缓存内容回应先前请求该内容的那个用户 |
| Connection         | Response | 连接状态通知                                                 | Close : 连接已经关闭<br />Keepalive : 连接保持 , 等待本次连接的后续请求 |
| Expired            | Response | Web服务器表明该实体将在什么时候过期                          | YYYY-MM-DD HH:MM:SS                                          |
| Location           | Response | 访问的对象已经被转移到别的位置了 , 应该到本头字段指向的地址获取 | http://mysite.com/another_url                                |
| Proxy-Authenticate | Response | 代理服务器响应浏览器 , 要求其提供代理身份验证信息            |                                                              |
| Server             | Response | 指明服务器的软件类型及版本                                   | Nginx/1.14                                                   |
| Etag               | Both     | 内容唯一标识 , 服务端要把服务器传来的Etage保留 , 在下次请求相同的 URL 时提交给服务器 . 服务器用 Etag值判断同一个 URL 的内容是否有变化 , 如果有变化则发送更新的内容给客户端 | 任何值                                                       |
| Via                | Both     | 列出从客户端到服务器或者相反方向的响应经过了哪些代理服务器 , 它们用什么协议 (和版本) 发送的请求 |    |

## HTTP常见错误代码

在每个 Response 的第1行中有一个整数状态码用于表达其对应 Request 的结果 , HTTP 除了约定该状态的表达方式 , 还约定了该状态的取值范围 , 约定的 5 类状态码如下 : 

- 1xx : 信息 ; 表明服务器已经收到 Request , 但需要进一步处理 , 请客户端等待
- 2xx : 成功 ; 处理成功
- 3xx : 重定向 ; 请求的地址已被重定向 , 需要客户端重新发起请求
- 4xx : 客户端错误 ; 请求中提交的参数或内容有错误
- 5xx : 服务器错误 ; 服务器处理请求时出错 , 一般本类错误需要联系服务器管理员处理

注意 : 1xx~5xx 的错误为 HTTP 标准错误 , 在网站开发中如需要定义自己的错误代码 , 则需要避开该范围

常见 HTTP 错误代码表如下 : 

| 代码 | 解释           | 代码 | 解释               | 代码 | 解释             |
| ---- | -------------- | ---- | ------------------ | ---- | ---------------- |
| 100  | 继续等待       | 200  | 正常完成并返回     | 204  | 无内容           |
| 206  | 部分内容被返回 | 301  | 已移动             | 304  | 未修改           |
| 305  | 必须使用代理   | 400  | 语法错误           | 401  | 未授权           |
| 402  | 需要付费访问   | 403  | 禁止访问           | 500  | 服务器异常错误   |
| 501  | 未执行         | 502  | 上游的其他来源错误 | 503  | 临时过载或维护中 |

## HTTP请求方法

HTTP 中常用的访问方式及其意义

| 访问方式的名称 | 意义                                                         |
| -------------- | ------------------------------------------------------------ |
| DELETE         | 从给定的地址中删除信息                                       |
| GET            | 从访问的地址中获取信息 , 即获取信息头 , 也获取信息体 . 这是互联网上最主要的一种 HTTP 访问方式 |
| HEAD           | 从访问的地址中获取信息 , 它与 GET 的区别是 : HEAD 只获取信息头 , 不获取信息体 . 在 Flask 路由中如果声明了 GET 访问方式 , 则无需显式地声明 HEAD 访问方式 |
| OPTIONS        | 为客户端提供一种查询 "本URL地址中有哪些可用的访问方式" 的方法 |
| POST           | 客户端通过 POST 方法向服务器提交新数据 , 服务器必须保证数据被完整地保存 , 并且服务器不允许出现重复的 POST 数据提交 , 这是 HTML 中通过表单提交数据所使用的 URL 访问方式 |
| PUT            | 与 POST 访问方法类似 , POST 也是一种使客户端可以向服务器提交数据的方式 , 但是 PUT 允许客户端提交重复主键的数据 , 当通过 PUT 访问方式在服务器中发现重复主键的数据时 , 它会用新提交的数据覆盖服务器中已有的数据 |