# Redis - 集群








<extoc></extoc>

## 介绍

Redis 集群是 Redis 提供的分布式数据库方案 , 集群通过分片 (sharding) 来进行数据共享 , 并提供复制和故障转移功能

## 节点

一个 Redis 集群通常由多个节点 (node) 组成 , 在刚开始的时候 , 每个节点都是相互独立的 , 它们都处于一个只包含自己的集群当中 , 要组建一个真正可工作的集群 , 我们必须将各个独立的节点连接起来 , 构成一个包含多个节点的集群

连接各个节点可以使用 `CLUSTER MEET` 命令来完成 , 命令格式如下 : 

```shell
CLUSTER MEET <ip> <port>
```

查看集群节点

```shell
CLUSTER NODES
```

一个节点就是一个运行在集群模式下的 Redis 服务器 , Redis 服务器在启动时会根据 `cluster-enabled` 配置选项是否为 `yes` 来决定是否开启服务器的集群模式

## 槽指派

Redis 集群通过分片的方式来保存数据库中的键值对 : 集群的整个数据被分为 16384 个槽 (slot) , 数据库中的每个键都属于这 16384 个槽的其中一个 , 集群中的每个节点可以处理 0 个或最多 16384 个槽

当数据库中的 16384 个槽都有节点在处理时 , 集群处于上线状态 , 相反的 , 如果数据库中公有任何一个槽没有得到处理 , 那么集群处于下线状态

通过向节点发送 `CLUSTER ADDSLOTS` 命令 , 我们可以将一个或多个槽指派给节点负责

```shell
CLUSTER ADDSLOTS <slot> [slot ...]
```

当客户端向节点发送与数据库键有关的命令时 , 接收命令的节点会计算出命令要处理的数据库键属于哪个槽 , 并检查这个槽是否指派给了自己 : 

- 如果键所在的槽正好就指派给了当前节点 , 那么节点直接执行这个命令
- 如果键所在的槽并没有指派给当前节点 , 那么节点会向客户端返回一个 MOVED 错误 , 指引客户端转向至正确的节点 , 并再次发送之前想要执行的命令

如果需要将已经指派给某个节点的槽改为指派给另一个节点 , 可以执行重新分片操作 

## 复制与故障转移

Redis 集群中的节点分为主节点 (master) 和从节点 (slave) , 其中主节点用于处理槽 , 而从节点则用于复制某个节点 , 并在被复制的主节点下线时 , 代替下线主节点继续处理命令请求

**设置从节点**

```shell
# 可以让接收命令的节点成为node_id所指定节点的从节点,并开始节点主从复制
CLUSTER REPLICATE <node_id>
```

**故障检测**

集群中的每个节点都会定期地向集群中的其他节点发送 PING 消息 , 以此来检测对方是否在线 , 如果接收 PING 消息的节点没有在规定的时间内 , 向发送 PING 消息的节点返回 PONG 消息 , 那么发送 PING 消息的节点就会将接收 PING 消息的节点标记为疑似下线 (probable fail , PFAIL)

集群中的各个节点会通过互相发消息的方式来交换集群中各个节点的状态信息 , 例如某个节点是处于在线状态 , 疑似下线状态 , 还是已下线状态

如果在一个集群里面 , 半数以上负责处理槽的主节点将某个主节点 x 报告为疑似下线 , 那么这个主节点 x 将被标记为已下线 , 将主节点 x 标记为下线的节点会向集群广播一条关于主节点 x 的 FALL 消息 , 所有收到这条 FALL 消息的节点都会立即将主节点 x 标记为已下线

**故障转移**

当一个从节点发现自己正在复制的主节点进入了已下线状态时 , 从节点将会开始对下线主节点进行故障转移 , 以下是故障转移的执行步骤 : 

1. 复制下线主节点的所有从节点里面 , 会有一个从节点被选中
2. 被选中的从节点会执行 SLAVEOF no one 命令 , 成为新的主节点
3. 新的主节点会撤销所有对已下线主节点的槽指派 , 并将这些槽全部指派给自己
4. 新的主节点向集群广播一条 PONG 消息 , 这条 PONG 消息可以让集群中的其他节点立即知道这个节点已经由从节点变成了主节点 , 并且这个主节点已经接管了原本由已下线节点负责处理的槽
5. 新的主节点开始接收和自己负责处理的槽有关的命令请求 , 故障转移完成

新的主节点是通过选举产生的

## 消息

集群中的各个节点通过发送和接收消息来进行通信 , 我们称发送消息的节点为发送者 , 接收消息的节点为接收者

节点发送的消息主要有以下五种 : 

- MEET 消息 : 当发送者接到客户端发送的 CLUSTER MEET 命令时 , 发送会向接收者发送 MEET 消息 , 请求接收者加入到发送者当前所处的集群里面
- PING 消息 : 集群里的每个节点默认每隔一秒钟就会从已知节点列表中随机选出五个节点 , 然后对五个节点中最长时间没有发送过 PING 消息的节点发送 PING 消息 , 以此来检测被选中的节点是否在线 ; 除此之外 , 如果节点 A 最后一次收到节点 B 发送的 PONG 消息的时间 , 距离当前时间已经超过了节点 A 的 `cluster-node-timeout` 选项设置时长的一半 , 那么节点 A 也会向节点 B 发送 PING 消息 , 这可以防止节点 A 因为长时间没有随机选中节点 B 作为 PING 消息的发送对象而导致节点 B 的信息更新滞后
- PONG 消息 : 当接收者收到发送者发来的 MEET 消息时 , 为了向发送者确认这条 MEET 消息或者 PING 消息已达到 , 接收者会向发送者返回一条 PONG 消息 , 另外 , 一个节点也可以通过向集群广播自己的 PONG 消息来让集群中的其他节点立即刷新关于这个节点的认识 , 例如当一次故障转移操作成功执行之后 , 新的主节点会向集群广播一条 PONG 消息 , 以此来让集群中的其他节点立即知道这个节点已经变成了主节点 , 并且接管了已经下线节点负责的槽
- FAIL 消息 : 当一个主节点 A 判断另一个主节点 B 已经进入 FAIL 状态时 , 节点 A 会向集群广播一条关于节点 B 的 FAIL 消息 , 所有收到这条消息的节点都会立即将节点 B 标记为已下线
- PUBLISH 消息 : 当节点接收到一个 PUBLISH 命令时 , 节点会执行这个命令 , 并向集群广播一条 PUBLIST 消息 , 所有接收到这条 PUBLISH 消息的节点都会执行相同的 PUBLISH 命令









